import zlib as zlib_original
from unittest.mock import patch

import aiohttp.compression_utils
import aiohttp.http_websocket
import pytest

import aiohttp_fast_zlib

try:
    from isal import (
        isal_zlib as expected_zlib,
    )
except ImportError:
    from zlib_ng import zlib_ng as expected_zlib


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION >= (3, 11),
    reason="Only works with aiohttp less than 3.11+",
)
def test_enable_disable_pre_311():
    """Test enable/disable."""
    assert aiohttp.http_websocket.zlib is zlib_original
    aiohttp_fast_zlib.enable()
    assert aiohttp.http_websocket.zlib is expected_zlib
    aiohttp_fast_zlib.disable()
    assert aiohttp.http_websocket.zlib is zlib_original
    aiohttp_fast_zlib.enable()
    assert aiohttp.http_websocket.zlib is expected_zlib
    aiohttp_fast_zlib.disable()


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION < (3, 11)
    or aiohttp_fast_zlib._AIOHTTP_VERSION >= (3, 12),
    reason="Only works with aiohttp 3.11.x",
)
def test_enable_disable_311():
    """Test enable/disable for aiohttp 3.11.x."""
    from aiohttp._websocket import writer

    assert writer.zlib is zlib_original
    aiohttp_fast_zlib.enable()
    assert writer.zlib is expected_zlib
    aiohttp_fast_zlib.disable()
    assert writer.zlib is zlib_original
    aiohttp_fast_zlib.enable()
    assert writer.zlib is expected_zlib
    aiohttp_fast_zlib.disable()


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION < (3, 11)
    or aiohttp_fast_zlib._AIOHTTP_VERSION >= (3, 12),
    reason="Only works with aiohttp 3.11.x",
)
def test_enable_disable_when_all_missing_311():
    """Test enable/disable for aiohttp 3.11.x when all fast libs are missing."""
    from aiohttp._websocket import writer

    with patch.object(aiohttp_fast_zlib, "best_zlib", zlib_original):
        assert writer.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert writer.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert writer.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert writer.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert writer.zlib is zlib_original


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION >= (3, 11),
    reason="Only works with aiohttp less than 3.11+",
)
def test_enable_disable_when_all_missing_pre_311():
    """Test enable/disable."""
    with patch.object(aiohttp_fast_zlib, "best_zlib", zlib_original):
        assert aiohttp.http_websocket.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert aiohttp.http_websocket.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert aiohttp.http_websocket.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert aiohttp.http_websocket.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert aiohttp.http_websocket.zlib is zlib_original


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION >= (3, 12),
    reason="Only works with aiohttp < 3.12",
)
def test_enable_disable_when_all_missing():
    """Test enable/disable when all fast libs are missing."""
    with patch.object(aiohttp_fast_zlib, "best_zlib", zlib_original):
        assert aiohttp.compression_utils.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert aiohttp.compression_utils.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert aiohttp.compression_utils.zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert aiohttp.compression_utils.zlib is zlib_original
        aiohttp_fast_zlib.disable()
        assert aiohttp.compression_utils.zlib is zlib_original


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION < (3, 12),
    reason="Only works with aiohttp >= 3.12",
)
def test_enable_disable_312_plus():
    """Test enable/disable for aiohttp 3.12+ with native set_zlib_backend."""
    from aiohttp.compression_utils import ZLibBackend

    # Test enable
    aiohttp_fast_zlib.enable()
    assert ZLibBackend._zlib_backend is expected_zlib

    # Test disable
    aiohttp_fast_zlib.disable()
    assert ZLibBackend._zlib_backend is zlib_original

    # Test enable again
    aiohttp_fast_zlib.enable()
    assert ZLibBackend._zlib_backend is expected_zlib

    # Clean up
    aiohttp_fast_zlib.disable()
    assert ZLibBackend._zlib_backend is zlib_original


@pytest.mark.skipif(
    aiohttp_fast_zlib._AIOHTTP_VERSION < (3, 12),
    reason="Only works with aiohttp >= 3.12",
)
def test_enable_disable_when_all_missing_312_plus():
    """Test enable/disable for aiohttp 3.12+ when all fast libs are missing."""
    from aiohttp.compression_utils import ZLibBackend

    # Store the original backend
    original_backend = ZLibBackend._zlib_backend

    with patch.object(aiohttp_fast_zlib, "best_zlib", zlib_original):
        # Test enable - should not change backend when best_zlib is zlib_original
        aiohttp_fast_zlib.enable()
        assert ZLibBackend._zlib_backend is original_backend

        # Test disable - should not change backend either
        aiohttp_fast_zlib.disable()
        assert ZLibBackend._zlib_backend is original_backend



"""Fast zlib module for aiohttp."""

__version__ = "0.3.0"

import contextlib
import importlib
import logging
import zlib as zlib_original
from typing import TYPE_CHECKING

import aiohttp

_LOGGER = logging.getLogger(__name__)

_AIOHTTP_SPLIT_VERSION = aiohttp.__version__.split(".")
_AIOHTTP_VERSION = (int(_AIOHTTP_SPLIT_VERSION[0]), int(_AIOHTTP_SPLIT_VERSION[1]))

if TYPE_CHECKING:
    best_zlib = zlib_original

try:
    from isal import isal_zlib as best_zlib  # type: ignore
except ImportError:
    try:
        from zlib_ng import zlib_ng as best_zlib  # type: ignore
    except ImportError:
        best_zlib = zlib_original

TARGETS = [
    "compression_utils",
    "http_writer",
    "http_writer",
    "http_parser",
    "multipart",
    "web_response",
]

_PATCH_WEBSOCKET_WRITER = False
_USE_NATIVE_ZLIB_BACKEND = False

# Check if aiohttp has native set_zlib_backend support (3.12+)
if _AIOHTTP_VERSION >= (3, 12):
    from aiohttp import set_zlib_backend

    _USE_NATIVE_ZLIB_BACKEND = True
elif _AIOHTTP_VERSION >= (3, 11):
    _PATCH_WEBSOCKET_WRITER = True
else:
    TARGETS.append("http_websocket")


def enable() -> None:
    """Enable fast zlib."""
    if best_zlib is zlib_original:
        _LOGGER.warning(
            "zlib_ng and isal are not available, falling back to zlib"
            ", performance will be degraded."
        )
        return

    if _USE_NATIVE_ZLIB_BACKEND:
        set_zlib_backend(best_zlib)
        return
    # Use patching for older versions
    _patch_modules()


def disable() -> None:
    """Disable fast zlib and restore the original zlib."""
    if _USE_NATIVE_ZLIB_BACKEND:
        # Use native aiohttp 3.12+ API
        set_zlib_backend(zlib_original)
        return
    # Use patching for older versions
    _unpatch_modules()


def _patch_modules() -> None:
    """Patch aiohttp modules to use best_zlib."""
    for location in TARGETS:
        try:
            importlib.import_module(f"aiohttp.{location}")
        except ImportError:
            continue
        if module := getattr(aiohttp, location, None):
            module.zlib = best_zlib
    if _PATCH_WEBSOCKET_WRITER:
        with contextlib.suppress(ImportError):
            mod = importlib.import_module("aiohttp._websocket.writer")
            mod.zlib = best_zlib  # type: ignore[attr-defined]


def _unpatch_modules() -> None:
    """Unpatch aiohttp modules to use original zlib."""
    for location in TARGETS:
        if module := getattr(aiohttp, location, None):
            module.zlib = zlib_original
    if _PATCH_WEBSOCKET_WRITER:
        with contextlib.suppress(ImportError):
            mod = importlib.import_module("aiohttp._websocket.writer")
            mod.zlib = zlib_original  # type: ignore[attr-defined]
